# --**--coding: utf-8--*--
"""CFCA Python SDK version 0.0.1"""

import os
import base64
from ctypes import *


my_path = os.path.dirname(os.path.abspath(__file__))


class CFCASignature:
    def __init__(self):
        so_file_path = os.path.join(my_path, "lib", "libSADK_Standard.so.3.4.1.2")
        self.so = cdll.LoadLibrary(so_file_path)
        self.res = self.so.Initialize()
        self.sign_data = c_char_p(b'')
        self.cert = c_char_p(b'')
        if self.res != 0:
            raise Exception("Init so file error")

    def get_sign(self, algorithm, source_data, pfx_file_path, pfx_password, hash_alg):
        """
        获取签名
        :param algorithm: (str) 算法
        :param source_data: (str) 待加密原文
        :param pfx_file_path: (str) pfx 文件地址
        :param pfx_password: (str) pfx 密码
        :param hash_alg: (str) 算法
        :return: (int) 状态码 (str)签名数据
        """
        algorithm = self._str2bytes(algorithm, 'ascii')
        source_data = self._str2bytes(source_data, 'ascii')
        pfx_file_path = self._str2bytes(pfx_file_path, 'ascii')
        pfx_password = self._str2bytes(pfx_password, 'ascii')
        hash_alg = self._str2bytes(hash_alg, 'ascii')

        source_size = len(source_data)
        base64_pkcs7_attached_signature = pointer(self.sign_data)
        res = self.so.SignData_PKCS7Attached(algorithm,
                                             source_data,
                                             source_size,
                                             pfx_file_path,
                                             pfx_password,
                                             hash_alg,
                                             base64_pkcs7_attached_signature)
        return res, self.sign_data.value

    def check_sign(self, base64_signature, algorithm):
        """
        验证签名
        :param base64_signature: (str) base64 格式的 signature
        :param algorithm: (str) 算法
        :return: 状态码 原文
        """
        sign_str = base64.b64decode(base64_signature)

        byte_size = c_int(0)
        p_byte_size = pointer(byte_size)

        p_cert = pointer(self.cert)

        source_data = c_char_p(b'')
        p_source_data = pointer(source_data)

        return_code = self.so.VerifyDataSignature_PKCS7Attached(algorithm, sign_str, p_cert, p_source_data, p_byte_size)
        return return_code, source_data.value[0:byte_size.value]

    def verify_certificate(self, certificate_path, verify_level):
        """
        验证证书
        :param certificate_path: (str) 证书路径，如有多个，用 | 进行分割
        :param verify_level: (int) 验证等级 1:验证证书时间有效性；2:验证证书是否被吊销；4:验证证书链完整性。以上标识位可以组合使用，例如传入7 就是全部都验证
        :return:
        """
        certificate_path = self._str2bytes(certificate_path, encoding='ascii')
        return_code = self.so.VerifyCertificate(self.cert.value, verify_level, certificate_path, None)
        return return_code

    def check_sign_cer(self, base64_signature, algorithm, certificate_path, verify_level):
        """
        校验签名和证书
        :param base64_signature: base64 格式的 sign
        :param algorithm: 算法
        :param certificate_path: 证书路径
        :param verify_level: 验证等级
        :return: 校验 sign 后的值、校验证书 code
        """
        _, _value = self.check_sign(base64_signature, algorithm)
        check_cer_code = self.verify_certificate(certificate_path, verify_level)
        return _value, check_cer_code

    @staticmethod
    def _str2bytes(string, encoding):
        return string.encode(encoding=encoding)
